@@ -48,6 +48,7 @@ module Distribution.Simple.Build
 
     -- * Handling of internal build tools
   , addInternalBuildTools
+  , addInternalBuildToolsFixed
   ) where
 
 import Distribution.Compat.Prelude
@@ -188,13 +189,15 @@ build_setupHooks
     -- dumped.
     dumpBuildInfo verbosity distPref (configDumpBuildInfo (configFlags lbi)) pkg_descr lbi flags
 
+    curDir <- absoluteWorkingDirLBI lbi
+
     -- Now do the actual building
     (\f -> foldM_ f (installedPkgs lbi) componentsToBuild) $ \index target -> do
       let comp = targetComponent target
           clbi = targetCLBI target
           bi = componentBuildInfo comp
           -- Include any build-tool-depends on build tools internal to the current package.
-          progs' = addInternalBuildTools pkg_descr lbi bi (withPrograms lbi)
+          progs' = addInternalBuildToolsFixed (Just curDir) pkg_descr lbi bi (withPrograms lbi)
           lbi' =
             lbi
               { withPrograms = progs'
@@ -376,17 +379,20 @@ repl_setupHooks
 
     internalPackageDB <- createInternalPackageDB verbosity lbi distPref
 
-    let lbiForComponent comp lbi' =
-          lbi'
-            { withPackageDB = withPackageDB lbi ++ [internalPackageDB]
-            , withPrograms =
-                -- Include any build-tool-depends on build tools internal to the current package.
-                addInternalBuildTools
-                  pkg_descr
-                  lbi'
-                  (componentBuildInfo comp)
-                  (withPrograms lbi')
-            }
+    let lbiForComponent comp lbi' = do
+          curDir <- absoluteWorkingDirLBI lbi'
+          return $
+            lbi'
+              { withPackageDB = withPackageDB lbi' ++ [internalPackageDB]
+              , withPrograms =
+                  -- Include any build-tool-depends on build tools internal to the current package.
+                  addInternalBuildToolsFixed
+                    (Just curDir)
+                    pkg_descr
+                    lbi'
+                    (componentBuildInfo comp)
+                    (withPrograms lbi')
+              }
         runPreBuildHooks :: LocalBuildInfo -> TargetInfo -> IO ()
         runPreBuildHooks lbi2 tgt =
           let inputs =
@@ -404,7 +410,7 @@ repl_setupHooks
       [ do
         let clbi = targetCLBI subtarget
             comp = targetComponent subtarget
-            lbi' = lbiForComponent comp lbi
+        lbi' <- lbiForComponent comp lbi
         preBuildComponent runPreBuildHooks verbosity lbi' subtarget
         buildComponent
           (mempty{buildCommonFlags = mempty{setupVerbosity = toFlag verbosity}})
@@ -421,7 +427,7 @@ repl_setupHooks
     -- REPL for target components
     let clbi = targetCLBI target
         comp = targetComponent target
-        lbi' = lbiForComponent comp lbi
+    lbi' <- lbiForComponent comp lbi
     preBuildComponent runPreBuildHooks verbosity lbi' target
     replComponent flags verbosity pkg_descr lbi' suffixHandlers comp clbi distPref
 
@@ -925,13 +931,18 @@ createInternalPackageDB verbosity lbi distPref = do
 --    directory environment variable for the current package to the current
 --    'progOverrideEnv', so that any programs configured from now on will be
 --    able to invoke these build tools.
-addInternalBuildTools
-  :: PackageDescription
+--
+--  NB: This function will be removed in the next Cabal major version
+--  (use addInternalBuildTools instead). This function is introduced solely for
+--  backporting in a PVP compliant way.
+addInternalBuildToolsFixed
+  :: Maybe (AbsolutePath (Dir Pkg))
+  -> PackageDescription
   -> LocalBuildInfo
   -> BuildInfo
   -> ProgramDb
   -> ProgramDb
-addInternalBuildTools pkg lbi bi progs =
+addInternalBuildToolsFixed mpwd pkg lbi bi progs =
   prependProgramSearchPathNoLogging
     internalToolPaths
     [pkgDataDirVar]
@@ -950,13 +961,27 @@ addInternalBuildTools pkg lbi bi progs =
                 buildDir lbi
                   </> makeRelativePathEx (toolName' </> toolName' <.> exeExtension (hostPlatform lbi))
       ]
-    mbWorkDir = mbWorkDirLBI lbi
-    rawDataDir = dataDir pkg
-    dataDirPath
-      | null $ getSymbolicPath rawDataDir =
-          interpretSymbolicPath mbWorkDir sameDirectory
-      | otherwise =
-          interpretSymbolicPath mbWorkDir rawDataDir
+
+    dataDirPath :: FilePath
+    dataDirPath =
+      case mpwd of
+        -- This is an absolute path, so if a process changes directory, it can still
+        -- find the datadir (#10717)
+        Just pwd -> interpretSymbolicPathAbsolute pwd (dataDir pkg)
+        -- This is just wrong, but implemented for PVP compliance..
+        Nothing -> interpretSymbolicPathCWD (dataDir pkg)
+
+{-# WARNING addInternalBuildTools "This function is broken, use addInternalBuildToolsFixed instead" #-}
+
+-- | A backwards compatible (broken) version of `addInternalBuildTools`, do not
+-- use this function. Use 'addInternalBuildToolsFixed' instead.
+addInternalBuildTools
+  :: PackageDescription
+  -> LocalBuildInfo
+  -> BuildInfo
+  -> ProgramDb
+  -> ProgramDb
+addInternalBuildTools = addInternalBuildToolsFixed Nothing
 
 -- TODO: build separate libs in separate dirs so that we can build
 -- multiple libs, e.g. for 'LibTest' library-style test suites
